/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew;

import de.bsvrz.dav.daf.main.Data;

/**
 * Kommunikationsparameter zur Kommunikation mit der Landesmeldestelle.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: LMSParameter.java 63650 2016-09-22 17:11:32Z gieseler $
 * 
 */
public class LMSParameter {

	/** Hostname oder IP-Adresse des Rechners der LMS. */
	private String lmsHostRechner = "";

	/** Benutzername am Rechners der LMS. */
	private String lmsHostNutzer = "";

	/** Passwort am Rechners der LMS. */
	private String lmsHostPasswort = "";

	/**
	 * Eigene ID f&uuml;r die zu versendenden Meldungen. Vorgabe f&uuml;r das
	 * Element 'DataProducer' (Datenproduzent) in der XML-Datei bei zu
	 * versendenden Meldungen.
	 */
	private String eigeneID = "";

	/**
	 * Pfad des Ausgangsverzeichnisses. Pfadname auf dem LMS-Rechner, in dem die
	 * zu versendenden Meldungen abgelegt werden sollen. Wenn der Wert nicht
	 * gesetzt ist, werden die Dateien ohne Verzeichniswechsel transferiert,
	 * d.h. in das Verzeichnis, in dem der FTP-Client nach der Anmeldung steht.
	 */
	private String ausgangsVerzeichnis = "";

	/**
	 * Pfad auf dem lokalen Rechner, in dem die Meldungen der LMS erwartet
	 * werden.
	 */
	private String eingangsVerzeichnis = "";

	/**
	 * Pfad des Sicherungsverzeichnisses. Pfadname auf dem lokalen Rechner, in
	 * dem fehlerhafte Dateien gespeichert werden.
	 */
	private String sicherungsVerzeichnis = "";

	/**
	 * Legt die Dauer (in Tagen) fest, wie lange die gesicherten Dateien im
	 * Sicherungsverzeichis gespeichert werden.
	 */
	private long sicherungsZeitTage = 5;

	/**
	 * Zeitintervall zur &Uuml;berpr&uuml;fung der Kommunikation. Die
	 * Kommunikation zwischen LMS und VRZ wird mit diesem Zyklus &uuml;berwacht.
	 * Der Wert gilt f&uuml;r die &Uuml;berwachung beider Richtungen.
	 * Standardwert: 5 min
	 */
	private long testIntervallKommunikation = 5 * 60 * 1000;

	/**
	 * Laufende Nummer der Meldung der letzten an die LMS versendeten Meldung.
	 */
	private long laufendeNummer;

	/**
	 * Aktuelle laufende Nummer der letzten an die LMS versendeten Meldung.
	 */
	private long aktuelleLaufendeNummer;

	/**
	 * Prefix des Dateinamens für die an die LMS zu sendende Datei.
	 */
	private String versandDateiNamePrefix;

	/**
	 * Definition der Versandart von Meldungen zur LMS.
	 */
	private int versandart;

	/**
	 * Versandzykluszeit bei Versandart 'zyklisch'.
	 */
	private long versandzyklus;

	/**
	 * KoordinatenRaster.
	 */
	private int koordinatenRaster;

	/**
	 * Aktualisiert die Parameter auf der Basis eines von DAV empfangenen
	 * Datensatzes.
	 *
	 * @param data
	 *            DAV-Datensatz
	 */
	public synchronized void aktualisieren(final Data data) {
		if (data == null) {
			throw new RuntimeException("Keine Parameter verfügbar");
		}

		lmsHostRechner = data.getTextValue("LMSHostRechner").getText();
		lmsHostNutzer = data.getTextValue("LMSHostNutzer").getText();
		lmsHostPasswort = data.getTextValue("LMSHostPasswort").getText();
		eigeneID = data.getTextValue("EigeneID").getText();
		versandDateiNamePrefix = data.getTextValue("VersandDateiNamePrefix").getText();
		ausgangsVerzeichnis = data.getTextValue("Ausgangsverzeichnis").getText();
		eingangsVerzeichnis = data.getTextValue("Eingangsverzeichnis").getText();
		sicherungsVerzeichnis = data.getTextValue("Sicherungsverzeichnis").getText();
		sicherungsZeitTage = data.getUnscaledValue("Sicherungszeit").longValue();
		testIntervallKommunikation = data.getUnscaledValue("TestIntervallKommunikation").longValue() * 60 * 1000;
		laufendeNummer = data.getUnscaledValue("LaufendeNummer").longValue();
		aktuelleLaufendeNummer = laufendeNummer;
		versandart = data.getUnscaledValue("Versandart").intValue();
		versandzyklus = data.getTimeValue("VersandZyklus").getMillis();
		koordinatenRaster = data.getUnscaledValue("KoordinatenRaster").intValue();
	}

	/**
	 * Gibt das parametrierte Ausgangsverzeichnis zur&uuml;ck.
	 *
	 * @return Ausgangsverzeichnis
	 */
	public String getAusgangsverzeichnis() {
		return ausgangsVerzeichnis;
	}

	/**
	 * Gibt die parametrierte eigene ID zur&uuml;ck.
	 *
	 * @return eigene ID
	 */
	public String getEigeneID() {
		return eigeneID;
	}

	/**
	 * Gibt das parametrierte Eingangsverzeichnis zur&uuml;ck.
	 *
	 * @return Eingangsverzeichnis
	 */
	public String getEingangsverzeichnis() {
		return eingangsVerzeichnis;
	}

	/**
	 * Setzt das Eingangsverzeichnis.
	 *
	 * @param verzeichnis
	 *            Eingangsverzeichnis
	 */
	public void setEingangsVerzeichnis(final String verzeichnis) {
		this.eingangsVerzeichnis = verzeichnis;
	}

	/**
	 * Gibt die aktuelle laufende Nummer f&uuml;r die zu versendenen
	 * TMC-Meldungen zur&uuml;ck.
	 *
	 * @return aktuelle laufende Nummer
	 */
	public long getLaufendeNummer() {
		return laufendeNummer;
	}

	/**
	 * Gibt die n&auml;chste zu verwendende laufende Nummer f&uuml;r die zu
	 * versendenen TMC-Meldungen zur&uuml;ck.
	 *
	 * @return n&auml;chste zu verwendende laufende Nummer
	 */
	public long getAktuelleLaufendeNummer() {
		return aktuelleLaufendeNummer;
	}

	/**
	 * Gibt die n&auml;chste zu verwendende laufende Nummer f&uuml;r die zu
	 * versendenen TMC-Meldungen zur&uuml;ck.
	 *
	 * @return n&auml;chste zu verwendende laufende Nummer
	 */
	public long getLaufendeNummerMeldung() {
		return ++aktuelleLaufendeNummer;
	}

	/**
	 * Gibt den parametrierten Nutzernamen am LMS-Host zur&uuml;ck.
	 *
	 * @return Nutzername
	 */
	public String getLMSHostNutzer() {
		return lmsHostNutzer;
	}

	/**
	 * Gibt das parametrierte Passwort am LMS-Host zur&uuml;ck.
	 *
	 * @return Passwort
	 */
	public String getLMSHostPasswort() {
		return lmsHostPasswort;
	}

	/**
	 * Gibt den parametrierten LMS-Host zur&uuml;ck.
	 *
	 * @return Hostname LMS-Host
	 */
	public String getLMSHostRechner() {
		return lmsHostRechner;
	}

	/**
	 * Gibt den Prefix des Dateinamens für die an die LMS zu sendende Datei
	 * zur&uuml;ck.
	 * 
	 * @return Prefix
	 */
	public String getVersandDateiNamePrefix() {
		return versandDateiNamePrefix;
	}

	/**
	 * Gibt das parametrierte Testintervall f&uuml;r die
	 * Kommunikationsverbindung zur&uuml;ck.
	 *
	 * @return Testintervall
	 */
	public long getTestIntervallKommunikation() {
		return testIntervallKommunikation;
	}

	/**
	 * Gibt die Versandart von Meldungen zur LMS zur&uuml;ck.
	 *
	 * @return Versandart
	 */
	public int getVersandart() {
		return versandart;
	}

	/**
	 * KoordinatenRaster.
	 * 
	 * @return KoordinatenRaster in m
	 */
	public int getKoordinatenRaster() {
		return koordinatenRaster;
	}

	/**
	 * Gibt das Flag zur&uuml;ck, ob der Versand spontan erfolgen soll.
	 * 
	 * @return <code>true</code> wenn spontaner Versadn, sonst
	 *         <code>false</code>
	 */
	public boolean isVersandSpontan() {
		return versandart == 0;
	}

	/**
	 * Gibt die Versandzykluszeit von Meldungen zur LMS zur&uuml;ck.
	 *
	 * @return Versandzykluszeit
	 */
	public long getVersandzyklus() {
		return versandzyklus;
	}

	/**
	 * Setzt das Ausgangsverzeichnis.
	 *
	 * @param verzeichnis
	 *            Ausgangsverzeichnis
	 */
	public void setAusgangsverzeichnis(final String verzeichnis) {
		ausgangsVerzeichnis = verzeichnis;
	}

	/**
	 * Setzt die aktuelle laufende Nummer f&uuml;r die zu versendenen
	 * RDS-Meldungen.
	 *
	 * @param laufendeNummer
	 *            neue aktuelle laufende Nummer
	 */
	public void setLaufendeNummer(final long laufendeNummer) {
		this.laufendeNummer = laufendeNummer;
	}

	/**
	 * Setzt den Nutzernamen am LMS-Host.
	 *
	 * @param nutzer
	 *            Nutzername
	 */
	public void setLMSHostNutzer(final String nutzer) {
		lmsHostNutzer = nutzer;
	}

	/**
	 * Setzt das Passwort am LMS-Host.
	 *
	 * @param passwort
	 *            Passwort
	 *
	 */
	public void setLMSHostPasswort(final String passwort) {
		lmsHostPasswort = passwort;
	}

	/**
	 * Setzt den LMS-Host.
	 *
	 * @param rechner
	 *            LMS-Rechner
	 */
	public void setLMSHostRechner(final String rechner) {
		lmsHostRechner = rechner;
	}

	/**
	 * Gibt den Pfad des Sicherungsverzeichnisses zur&uuml;ck.
	 * 
	 * @return Sicherungsverzeichnis
	 */
	public String getSicherungsVerzeichnis() {
		return sicherungsVerzeichnis;
	}

	/**
	 * Gibt die Sicherungszeit zur&uuml;ck.
	 * 
	 * @return Sicherungszeit
	 */
	public long getSicherungsZeitTage() {
		return sicherungsZeitTage;
	}

	/**
	 * Setzt die laufende Nummer zur&uuml;ck.
	 */
	public void resetLaufendeNummer() {
		laufendeNummer = aktuelleLaufendeNummer;
	}
}
