/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic.elemente;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.daten.tic.AbstractTIC3InfoElement;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3InfoAttribut;

/**
 * TIC3-Ereignisdaten.
 * <p>
 * Entspricht TIC3 Typ 'TrafficAndTravelEventType' (ohne SaeRdsEvent).
 * </p>
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: TIC3Event.java 64047 2016-10-27 15:42:01Z gieseler $
 */
public class TIC3Event extends AbstractTIC3InfoElement {

	/**
	 * TIC3 Info XML Element: Event (EVT), Event codes with additional
	 * information for different event coding methods, such as TIC3, TMC, or SAE
	 * RDS.
	 */
	public static final String XML_ELEMENT_EVENT = "Event";

	/**
	 * DAV-Attribut.
	 */
	public static final String DAV_ATTR_EVENT = "TIC3Ereignis";

	/**
	 * Beschreibung.
	 * <p>
	 * Beschreibung aus Wert des 'description'-Attributes.
	 * </p>
	 */
	private TIC3InfoAttribut beschreibung = new TIC3InfoAttribut("description", "Beschreibung");

	/**
	 * TIC3 Info XML Element: TmcEvent (TMC), A list of one or more TMC event
	 * codes with additional information which are equivalent to "TIC3 event"
	 * codes, where TMC is defined in the TMC standard ISO 14819-2.
	 */
	private List<TIC3TMCEvent> tmcEvents = new ArrayList<>();

	/**
	 * TIC3 Info XML Element: Tic3Event (T3E), A list of one or more TIC3
	 * traffic and travel event codes and their attributes, where the TIC3 event
	 * codes are defined based on a combination of standards such asTMC or SAE
	 * RDS.
	 */
	private List<TIC3TICEvent> ticEvents = new ArrayList<>();

	/**
	 * Erzeugt ein neues Objekt.
	 */
	public TIC3Event() {
		super(XML_ELEMENT_EVENT, DAV_ATTR_EVENT);
		addAttribut(beschreibung);
	}

	@Override
	public void publizieren(final Data davdata, final LMSParameter parameter) throws TIC3Exception {
		super.publizieren(davdata, parameter);

		final Data ddata = getLocalDavData(davdata);

		if (tmcEvents.size() > 0) {
			final Data.Array array = ddata.getArray(TIC3TMCEvent.DAV_ATTR_TMCEVENT);
			array.setLength(tmcEvents.size());
			for (int i = 0; i < tmcEvents.size(); i++) {
				tmcEvents.get(i).publizieren(array.getItem(i), parameter);
			}
		}

		if (ticEvents.size() > 0) {
			final Data.Array array = ddata.getArray(TIC3TICEvent.DAV_ATTR_TICEVENT);
			array.setLength(ticEvents.size());
			for (int i = 0; i < ticEvents.size(); i++) {
				ticEvents.get(i).publizieren(array.getItem(i), parameter);
			}
		}
	}

	@Override
	public void einlesenXML(final Node node) throws TIC3Exception {
		super.einlesenXML(node);
		final NodeList nl = node.getChildNodes();

		for (int i = 0; i < nl.getLength(); i++) {
			final Node n = nl.item(i);
			if (n.getNodeName().equals(TIC3TMCEvent.XML_ELEMENT_TMCEVENT)) {
				final TIC3TMCEvent event = new TIC3TMCEvent();
				event.einlesenXML(n);
				tmcEvents.add(event);
			}
			if (n.getNodeName().equals(TIC3TICEvent.XML_ELEMENT_TICEVENT)) {
				final TIC3TICEvent event = new TIC3TICEvent();
				event.einlesenXML(n);
				ticEvents.add(event);
			}
		}
	}

	@Override
	public Element erzeugeDocumentElement(final Document doc, final Element parent) throws TIC3Exception {
		final Element element = super.erzeugeDocumentElement(doc, parent);

		if (tmcEvents.size() > 0) {
			for (TIC3TMCEvent ev : tmcEvents) {
				ev.erzeugeDocumentElement(doc, element);
			}
		}

		return element;
	}

	@Override
	public String toString() {
		return this.getClass().getSimpleName();
	}

	/**
	 * Gibt die Liste der TMC-Ereignisse zur&uuml;ck.
	 * 
	 * @return List von {@link TIC3TMCEvent}
	 */
	public List<TIC3TMCEvent> getTMCEreignisse() {
		return tmcEvents;
	}

	/**
	 * F&uuml;gt ein Ereignis hinzu.
	 * 
	 * @param event
	 *            {@link TIC3TMCEvent}
	 */
	public void addTMCEreignis(final TIC3TMCEvent event) {
		tmcEvents.add(event);
	}
}
